﻿' .NET 8 による
' 中央精機製ステージコントローラ QT-A / QT-B と通信するためのサンプルです。
' サンプルのため最低限の記述を心がけています。
' エラー処理、実装方法など開発言語についてはお使いの開発環境のヘルプなどをご参照いただくか、開発言語メーカへお問い合わせください。
'
' 【動作確認環境】
' ・ OS: Windows 10 22H2 / Windows 11 23H2
' ・ Visual C# 2022
'
' 【前提条件】
' ・ このサンプルでは 2 軸のコントローラを動作させます。
' ・ コントローラとステージが正しく接続され、正常に動作する設定になっていることとします。
' ・ コントローラはレスポンスなし (X:0) を想定しています。
' ・ プロジェクトに System.IO.Ports NuGet パッケージがインストールされているものとします。
'    System.IO.Ports Nuget パッケージが未インストールの場合、インストールしてください。
'    1. ソリューションエクスプローラでプロジェクトを右クリック > [NuGet パッケージの管理] を選択する
'    2. [パッケージソース] を "NuGet" にし、検索ボックスに "System.IO.Ports" と入力する
'    3. 一覧から "System.IO.Ports" を選択し、[インストール] をクリックする'
'
' 【サンプルコードの内容】
' 1. System.IO.Ports.SerialPort クラスのインスタンスを作成する
' 2. 通信ポートの設定を行う
' 3. 通信ポートを開く
' 4. QT-A / QT-B との RS-232C 通信を行う
'    - A 軸に接続されたステージを +1000 パルス移動開始します。
'    - 100 ms 待ちます。
'    - Q コマンドを送信し、返答された内容をコンソールに出力します。
' 5. 通信ポートを閉じる
'    ※ System.IO.Ports.SerialPort クラスのインスタンスを作成の際に Using ステートメントを使用している場合、
'    ※ Using の終了時に作成されたインスタンスは自動的に破棄され、その際に通信ポートは自動的に閉じられます。
'
' 【使用方法】
' 通信ポートの設定に合わせ、クラス内のフィールドを設定します。
' 「設定箇所 - ここから」～「設定箇所 - ここまで」の間をコントローラに合わせ変更します。
Imports System.IO.Ports

Module Program
    ' <<<<< 設定箇所 - ここから >>>>>
    ' 通信ポートの番号を指定します。
    ' 通信ポート番号はデバイスマネージャなどで確認します。
    Private Const PortName As String = "COM3"

    ' 通信ポートのボーレート 4800, 9600, 19200, (QT-B のみ) 38400 の何れかを指定します。
    ' お使いのコントローラの設定と併せてください。
    Private Const Baudrate As Integer = 9600
    'Private Const Baudrate As Integer = 4800
    'Private Const Baudrate As Integer = 19200
    'Private Const Baudrate As Integer = 38400 ' QT-B のみ

    ' 通信ポートのデータビット長 8 か 7 bit を指定する。
    ' お使いのコントローラの設定と併せてください。
    Private Const DataBits As Integer = 8
    'Private Const DataBits As Integer = 7

    ' 通信ポートのパリティ Parity.None (パリティなし),　Parity.Odd (奇数パリティ), Parity.Even (偶数パリティ) の何れかを指定する。
    ' お使いのコントローラの設定と併せてください。
    Private Const Parity As Parity = Parity.None
    'Private Const Parity As Parity = Parity.Odd
    'Private Const Parity As Parity = Parity.Even

    ' 通信ポートのストップビット長 StopBits.One, StopBits.Two の何れかを指定する。
    ' お使いのコントローラの設定と併せてください。
    Private Const StopBits As StopBits = StopBits.One
    'Private Const StopBits As StopBits = StopBits.Two

    ' コントローラとの通信で使用するデリミタを設定します。
    ' お使いのコントローラの設定と併せてください。
    ' vbCrLf: CR+LF、vbCr: CR、vbLf: LF
    Private Const Delimiter As String = vbCrLf
    'Private Const Delimiter As String = vbCr
    'Private Const Delimiter As String = vbLf

    ' <<<<< 設定箇所 - ここまで >>>>>

    Sub Main(args As String())
        ' シリアルポートクラスのインスタンスを作成します。
        ' Using ステートメントから抜ける箇所で自動的に破棄 (Dispose) されます。この時通信ポートが閉じられます。
        ' Using ステートメントを利用しない状況では明示的に Dispose を呼び出す必要があります。
        ' Using ステートメント: https://learn.microsoft.com/ja-jp/dotnet/visual-basic/language-reference/statements/using-statement
        Using Port = New SerialPort(PortName, Baudrate, Parity, DataBits, StopBits)
            ' お使いの通信ポートによっては追加の設定が必要になります
            ' 詳細: https://learn.microsoft.com/ja-jp/dotnet/api/system.io.ports.serialport?view=netframework-4.8
            With Port
                .NewLine = Delimiter
                .ReadTimeout = 1000
                .WriteTimeout = 1000
            End With

            ' シリアル通信ポートを開きます。
            Port.Open()

            ' MGO コマンドを送信
            Dim cmd = "MGO:A+1000"
            Port.WriteLine(cmd)
            Console.WriteLine($"通信ポート {PortName} にコマンド {cmd} を送信しました。")

            ' この部分は移動がある程度進むのを待つためのものなのでコントローラとの通信とは関係ありません。
            Task.Delay(100).Wait()

            ' Q コマンドを送信し、返答を受信します。
            cmd = "Q:A0"
            Port.WriteLine(cmd)
            Console.WriteLine($"通信ポート {PortName} にコマンド {cmd} を送信しました。")
            Dim readed = Port.ReadLine()
            Console.WriteLine($"通信ポート {PortName} からデータ {readed} を受信しました。")

            ' この部分は受信した文字の処理なのでコントローラとの通信には関係ありません。
            Dim pos As Integer
            Dim stat As Char
            If Int32.TryParse(readed.AsSpan(0, 9), pos) = False Then
                ' 位置を整数に変換できない場合、意図的に QT-A / QT-B で表現できない Int32.MinValue を入れています。
                pos = Int32.MinValue
            End If
            stat = readed.Last()

            Console.WriteLine($"A 軸: {pos} + {stat}")
        End Using
    End Sub
End Module
